@@ -57,9 +57,9 @@ def parse_cmdlineoptions(arglist):
 		  "include=", "include-filelist=", "include-filelist-stdin",
 		  "include-globbing-filelist=", "include-regexp=",
 		  "list-at-time=", "list-changed-since=", "list-increments",
-		  "list-increment-sizes", "never-drop-acls",
+		  "list-increment-sizes", "never-drop-acls", "no-acls",
 		  "no-compare-inode", "no-compression",
-		  "no-compression-regexp=", "no-file-statistics",
+		  "no-compression-regexp=", "no-eas", "no-file-statistics",
 		  "no-hard-links", "null-separator",
 		  "override-chars-to-quote=", "parsable-output",
 		  "print-statistics", "remote-cmd=", "remote-schema=",
@@ -111,10 +111,12 @@ def parse_cmdlineoptions(arglist):
 			action = "list-increments"
 		elif opt == '--list-increment-sizes': action = 'list-increment-sizes'
 		elif opt == "--never-drop-acls": Globals.set("never_drop_acls", 1)
+		elif opt == "--no-acls": Globals.set("acls_active", 0)
 		elif opt == "--no-compare-inode": Globals.set("compare_inode", 0)
 		elif opt == "--no-compression": Globals.set("compression", None)
 		elif opt == "--no-compression-regexp":
 			Globals.set("no_compression_regexp_string", arg)
+		elif opt == "--no-eas": Globals.set("eas_active", 0)
 		elif opt == "--no-file-statistics": Globals.set('file_statistics', 0)
 		elif opt == "--no-hard-links": Globals.set('preserve_hardlinks', 0)
 		elif opt == "--null-separator": Globals.set("null_separator", 1)
@@ -254,8 +256,8 @@ def Backup(rpin, rpout):
 	backup_set_rbdir(rpin, rpout)
 	backup_set_fs_globals(rpin, rpout)
 	if Globals.chars_to_quote: rpout = backup_quoted_rpaths(rpout)
-	backup_final_init(rpout)
 	init_user_group_mapping(rpout.conn)
+	backup_final_init(rpout)
 	backup_set_select(rpin)
 	if prevtime:
 		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
@@ -346,45 +348,48 @@ def backup_get_mirrortime():
 def backup_final_init(rpout):
 	"""Open the backup log and the error log, create increments dir"""
 	global prevtime
-	prevtime = backup_get_mirrortime()
-	need_check = checkdest_need_check(rpout)
 	if Log.verbosity > 0:
 		Log.open_logfile(Globals.rbdir.append("backup.log"))
+	checkdest_if_necessary(rpout)
+	prevtime = backup_get_mirrortime()
 	ErrorLog.open(Time.curtimestr, compress = Globals.compression)
-	if need_check:
-		Log("Previous backup seems to have failed, regressing "
-			"destination now.", 2)
-		rpout.conn.regress.Regress(rpout)
 	inc_base = Globals.rbdir.append_path("increments")
 	if not inc_base.lstat(): inc_base.mkdir()
 
 def backup_set_fs_globals(rpin, rpout):
 	"""Use fs_abilities to set the globals that depend on filesystem"""
-	def update_bool_global(attr, bool):
-		"""If bool is not None, update Globals.attr accordingly"""
-		if Globals.get(attr) is None:
-			SetConnections.UpdateGlobal(attr, bool)
+	def update_triple(src_support, dest_support, attr_triple):
+		"""Update global settings for feature based on fsa results"""
+		active_attr, write_attr, conn_attr = attr_triple
+		if Globals.get(active_attr) == 0: return # don't override 0
+		for attr in attr_triple: SetConnections.UpdateGlobal(attr, None)
+		if not src_support: return # if source doesn't support, nothing
+		SetConnections.UpdateGlobal(active_attr, 1)
+		rpin.conn.Globals.set_local(conn_attr, 1)
+		if dest_support:
+			SetConnections.UpdateGlobal(write_attr, 1)
+			rpout.conn.Globals.set_local(conn_attr, 1)
 
 	src_fsa = rpin.conn.fs_abilities.get_fsabilities_readonly('source', rpin)
 	Log(str(src_fsa), 3)
 	dest_fsa = rpout.conn.fs_abilities.get_fsabilities_readwrite(
 		'destination', Globals.rbdir, 1, Globals.chars_to_quote)
 	Log(str(dest_fsa), 3)
-	if Globals.never_drop_acls and not dest_fsa.acls:
+
+	update_triple(src_fsa.eas, dest_fsa.eas,
+				  ('eas_active', 'eas_write', 'eas_conn'))
+	update_triple(src_fsa.acls, dest_fsa.acls,
+				  ('acls_active', 'acls_write', 'acls_conn'))
+	update_triple(src_fsa.resource_forks, dest_fsa.resource_forks,
+				  ('resource_forks_active', 'resource_forks_write',
+				   'resource_forks_conn'))
+	if Globals.never_drop_acls and not Globals.acls_active:
 		Log.FatalError("--never-drop-acls specified, but ACL support\n"
 					   "disabled on destination filesystem")
 
-	update_bool_global('read_acls', src_fsa.acls)
-	update_bool_global('read_eas', src_fsa.eas)
-	update_bool_global('read_resource_forks', src_fsa.resource_forks)
-
 	SetConnections.UpdateGlobal('preserve_hardlinks', dest_fsa.hardlinks)
 	SetConnections.UpdateGlobal('fsync_directories', dest_fsa.fsync_dirs)
 	SetConnections.UpdateGlobal('change_ownership', dest_fsa.ownership)
-	update_bool_global('write_acls', Globals.read_acls and dest_fsa.acls)
-	update_bool_global('write_eas', Globals.read_eas and dest_fsa.eas)
-	update_bool_global('write_resource_forks',
-					   Globals.read_resource_forks and dest_fsa.resource_forks)
 	SetConnections.UpdateGlobal('chars_to_quote', dest_fsa.chars_to_quote)
 	if Globals.chars_to_quote:
 		for conn in Globals.connections:
@@ -431,6 +436,7 @@ def Restore(src_rp, dest_rp, restore_as_of = None):
 	if not restore_root_set: assert restore_set_root(src_rp)
 	restore_check_paths(src_rp, dest_rp, restore_as_of)
 	restore_set_fs_globals(dest_rp)
+	init_user_group_mapping(dest_rp.conn)
 	src_rp = restore_init_quoting(src_rp)
 	restore_check_backup_dir(restore_root, src_rp, restore_as_of)
 	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
@@ -438,7 +444,6 @@ def Restore(src_rp, dest_rp, restore_as_of = None):
 		try: time = Time.genstrtotime(restore_timestr, rp = inc_rpath)
 		except Time.TimeException, exc: Log.FatalError(str(exc))
 	else: time = src_rp.getinctime()
-	init_user_group_mapping(dest_rp.conn)
 	restore_set_select(restore_root, dest_rp)
 	restore_start_log(src_rp, dest_rp, time)
 	restore.Restore(restore_root.new_index(restore_index),
@@ -457,9 +462,16 @@ def restore_init_quoting(src_rp):
 
 def restore_set_fs_globals(target):
 	"""Use fs_abilities to set the globals that depend on filesystem"""
-	def update_bool_global(attr, bool):
-		"""If bool is not None, update Globals.attr accordingly"""
-		if Globals.get(attr) is None: SetConnections.UpdateGlobal(attr, bool)
+	def update_triple(src_support, dest_support, attr_triple):
+		"""Update global settings for feature based on fsa results"""
+		active_attr, write_attr, conn_attr = attr_triple
+		if Globals.get(active_attr) == 0: return # don't override 0
+		for attr in attr_triple: SetConnections.UpdateGlobal(attr, None)
+		if not dest_support: return # if dest doesn't support, do nothing
+		SetConnections.UpdateGlobal(active_attr, 1)
+		target.conn.Globals.set_local(conn_attr, 1)
+		target.conn.Globals.set_local(write_attr, 1)
+		if src_support: Globals.rbdir.conn.Globals.set_local(conn_attr, 1)
 
 	target_fsa = target.conn.fs_abilities.get_fsabilities_readwrite(
 		'destination', target, 0)
@@ -467,16 +479,18 @@ def restore_set_fs_globals(target):
 	mirror_fsa = Globals.rbdir.conn.fs_abilities.get_fsabilities_restoresource(
 		Globals.rbdir)
 	Log(str(mirror_fsa), 3)
-	if Globals.never_drop_acls and not target_fsa.acls:
+
+	update_triple(mirror_fsa.eas, target_fsa.eas,
+				  ('eas_active', 'eas_write', 'eas_conn'))
+	update_triple(mirror_fsa.acls, target_fsa.acls,
+				  ('acls_active', 'acls_write', 'acls_conn'))
+	update_triple(mirror_fsa.resource_forks, target_fsa.resource_forks,
+				  ('resource_forks_active', 'resource_forks_write',
+				   'resource_forks_conn'))
+	if Globals.never_drop_acls and not Globals.acls_active:
 		Log.FatalError("--never-drop-acls specified, but ACL support\n"
 					   "disabled on destination filesystem")
 
-	update_bool_global('read_acls', target_fsa.acls)
-	update_bool_global('write_acls', target_fsa.acls)
-	update_bool_global('read_eas', target_fsa.eas)
-	update_bool_global('write_eas', target_fsa.eas)
-	update_bool_global('read_resource_forks', target_fsa.resource_forks)
-	update_bool_global('write_resource_forks', target_fsa.resource_forks)
 	SetConnections.UpdateGlobal('preserve_hardlinks', target_fsa.hardlinks)
 	SetConnections.UpdateGlobal('change_ownership', target_fsa.ownership)
 
@@ -565,6 +579,9 @@ def restore_set_root(rpin):
 		parent_dir = rpath.RPath(rpin.conn, "/".join(pathcomps[:i]))
 		if (parent_dir.isdir() and
 			"rdiff-backup-data" in parent_dir.listdir()): break
+		print "##########", parent_dir.path, Globals.restrict_path
+		if parent_dir.path == rpin.conn.Globals.get('restrict_path'):
+			return None
 		i = i-1
 	else: return None
 
@@ -662,7 +679,9 @@ def ListChangedSince(rp):
 	restore_check_backup_dir(restore_root)
 	mirror_rp = restore_root.new_index(restore_index)
 	inc_rp = mirror_rp.append_path("increments", restore_index)
-	restore.ListChangedSince(mirror_rp, inc_rp, rest_time)
+	for rorp in rp.conn.restore.ListChangedSince(mirror_rp, inc_rp, rest_time):
+		# This is a hack, see restore.ListChangedSince for rational
+		print rorp.index[0]
 
 
 def ListAtTime(rp):
@@ -673,7 +692,8 @@ def ListAtTime(rp):
 	restore_check_backup_dir(restore_root)
 	mirror_rp = restore_root.new_index(restore_index)
 	inc_rp = mirror_rp.append_path("increments", restore_index)
-	restore.ListAtTime(mirror_rp, inc_rp, rest_time)
+	for rorp in rp.conn.restore.ListAtTime(mirror_rp, inc_rp, rest_time):
+		print rorp.get_indexpath()
 	
 
 def CheckDest(dest_rp):
@@ -695,8 +715,9 @@ def CheckDest(dest_rp):
 def checkdest_need_check(dest_rp):
 	"""Return None if no dest dir found, 1 if dest dir needs check, 0 o/w"""
 	if not dest_rp.isdir() or not Globals.rbdir.isdir(): return None
-	if Globals.rbdir.listdir() == ['chars_to_quote']:
-		# This may happen the first backup just after we test for quoting
+	for filename in Globals.rbdir.listdir():
+		if filename not in ['chars_to_quote', 'backup.log']: break
+	else: # This may happen the first backup just after we test for quoting
 		return None
 	curmirroot = Globals.rbdir.append("current_mirror")
 	curmir_incs = restore.get_inclist(curmirroot)
