@@ -20,7 +20,7 @@
 """Functions to make sure remote requests are kosher"""
 
 import sys, tempfile
-import Globals, Main, rpath
+import Globals, Main, rpath, log
 
 class Violation(Exception):
 	"""Exception that indicates an improper request has been received"""
@@ -125,19 +125,23 @@ def set_allowed_requests(sec_level):
 			["C.make_file_dict",
 			 "rpath.ea_get",
 			 "rpath.acl_get",
+			 "rpath.setdata_local",
 			 "log.Log.log_to_file",
 			 "os.getuid",
 			 "os.listdir",
 			 "Time.setcurtime_local",
 			 "rpath.gzip_open_local_read",
 			 "rpath.open_local_read",
-			 "Hardlink.initialize_dictionaries"])
+			 "Hardlink.initialize_dictionaries",
+			 "user_group.uid2uname",
+			 "user_group.gid2gname"])
 		if sec_level == "read-only":
 			allowed_requests.extend(
 				["fs_abilities.get_fsabilities_readonly",
 				 "fs_abilities.get_fsabilities_restoresource",
 				 "restore.MirrorStruct.set_mirror_and_rest_times",
 				 "restore.MirrorStruct.initialize_rf_cache",
+				 "restore.MirrorStruct.close_rf_cache",
 				 "restore.MirrorStruct.get_diffs",
 				 "backup.SourceStruct.get_source_select",
 				 "backup.SourceStruct.set_source_select",
@@ -166,7 +170,9 @@ def set_allowed_requests(sec_level):
 			 "Globals.postset_regexp_local",
 			 "Globals.set_select",
 			 "backup.SourceStruct.set_session_info",
-			 "backup.DestinationStruct.set_session_info"])
+			 "backup.DestinationStruct.set_session_info",
+			 "user_group.init_user_mapping",
+			 "user_group.init_group_mapping"])
 
 def vet_request(request, arglist):
 	"""Examine request for security violations"""
@@ -177,7 +183,7 @@ def vet_request(request, arglist):
 			if isinstance(arg, rpath.RPath): vet_rpath(arg)
 	if security_level == "all": return
 	if request.function_string in allowed_requests: return
-	if request.function_string == "Globals.set":
+	if request.function_string in ("Globals.set", "Globals.set_local"):
 		if Globals.server and arglist[0] not in disallowed_server_globals:
 			return
 	raise Violation("\nWarning Security Violation!\n"
@@ -199,7 +205,3 @@ def vet_rpath(rpath):
 							"Request to handle path %s\n"
 							"which doesn't appear to be within "
 							"restrict path %s.\n" % (normalized, restrict))
-
-			 
-		   
-			
